@@ -7,13 +7,11 @@ check.for.NA.in.trait <- function(trait=NULL)
        ## check for NA's in trait
         indxNA <- which(is.na(trait))
         if(length(indxNA)==0){
-          indxNA <- NA
+          indxNA <- NULL
         } else {
           ## place in reverse order
           indxNA <- sort(indxNA, decreasing = TRUE)
-message(cat("\n\n WARNING!!!! The phenotypic data in rows ", indxNA, " either have missing trait data "))
-message("             and/or missing explanatory variable values. These rows of data have ")
-message(cat("             been removed from the analysis.  \n"))
+message(cat("\n\n The phenotypic data in rows ", indxNA, " have missing trait data but their other data are still included.  "))
           if(any(is.na(indxNA))){
             message("Error:  (internal).  indxNA contains NA values. ")
             message(" AM has terminated with errors. ")
